/* $Id: verbs.t 1.15 2001/08/16 19:18:52 mol Exp $
 * --------------------------------------------------------------
 * Verb definitions for Aayela 1.2
 *
 * This file is a part of the source distribution of
 *
 * Aaylea - an Interactive Vision, release 1.2
 *
 * Copyright 1996-2001 by Magnus Olsson (mol@pobox.com).
 * All rights reserved.
 *
 * THIS CODE MAY BE DISTRIBUTED FREELY AS LONG AS IT IS NOT MODIFIED
 * IN ANY WAY, NO FEE IS CHARGED EXCEPT FOR DISTRIBUTION COST, AND
 * THIS COPYRIGHT NOTICE IS NOT ALTERED OR DELETED.
 * You may use the code, classes and algorithms as part of your
 * own programs as long as the following conditions are met:
 *
 * 1) No use of the characters, plot or text of the game
 *    "Aayela" are used in the derivative work (i.e. you may
 *    write another game set in darkness using this code, but
 *    you may not publish an "improved" version of Aayela)
 *    without the author's permission.
 * 2) The use of this code, and its origin, are acknowledged in 
 *    the derivative work.
 *
 * This version is tested with the Linux TADS compiler v2.5.5
 * --------------------------------------------------------------
 */

#include "d_std.t"

/*
 * Make 'inside' a synonym for 'in'
 */
modify inPrep
    preposition = ''
;

/*
 * We must be able to parse 'away' as a preposition (though it's an adverb)
 */
awayPrep : Prep
    preposition = ''
    sdesc = ""
;
    
/*
 * Make 'feel' a synonym for 'touch'
 */
modify touchVerb
    verb = '' '' '' '' ''
;

/*
 * Make 'taste', 'nibble' and 'bite' synonyms for 'eat'
 */
modify eatVerb
    verb = '' '' '' '' '' '' '' '' '' '' '' '' '' ''
;

/*
 * A new magic word
 */
queltuyaVerb : deepverb, darkVerb
    verb = 'queltuya'
    sdesc = "queltuya"
    action(actor) = {
        if (stone.isIn(endgame) and actor.isIn(endgame))
	    stone.queltuya(actor);
	else 
	    "  .";
    }
;

/*
 * And some old magic words, just for fun
 */
xyzzyVerb : deepverb, darkVerb
    verb = 'xyzzy' 'plugh'
    sdesc= "xyzzy"
    action(actor) = {
        "       , 
                 
         ... ";
    }
;

magicVerb : deepverb, darkVerb
    verb = 'yoho' 'necken-sway'
    	   'abracadabra' 'hocus' 'pocus' 'shazam' '' '' '' ''
    sdesc= ""
    action(actor) = {
        ",     ? ";
    }
;

frotzVerb : deepverb, darkVerb
    verb = 'frotz'
    sdesc = "frotz"
    action(actor) = {
        magicVerb.action(actor);
    }
    doAction = 'Frotz'
;

modify thing
    verDoFrotz(actor) = {
        magicVerb.action(actor);
    }
;

modify sayVerb
    isDarkVerb = true
    validDo(actor, obj, seqno) = {
        return true;
    }
    action(actor) = {
        "   ,    ( \"\", 
	       \" \").
	        - -,     
	      (\",   \").";
	      abort;
    }
    doDefault(actor, prep, io) = {
    	if (objwords(1) = ['A']) {
            global.allMessage := '    ""   .';
	    return [];
	}
	pass doDefault;
    }
;

modify thing
    verDoSay(actor) = {
        sayVerb.action(actor);
    }
;    
        
modify strObj
    doSay(actor) = {
        if (find(['XYZZY' 'PLUGH' 'ABRACADABRA' 'HOCUS' 'POCUS' 'SHAZAM' '' '' '' ''],
	         upper(self.value)))
	    xyzzyVerb.action(actor);
	else if (upper(self.value) = 'QUELTUYA')
	    queltuyaVerb.action(actor);
	else
	    sayVerb.action(actor);
    }
;
    
/*
 * Make 'dig' work without an indirect object, and in the dark
 */
replace digVerb : deepverb
    verb = '' '' '' '' '' ''
    isDarkReachableVerb = true
    sdesc = ""
    doAction = 'DigIn'
    doDefault(actor, prep, io) = {
	local lst := inherited.doDefault(actor, prep, io);
	if (actor.location.floorobj <> nil)
	    lst += actor.location.floorobj;
	return lst;
    }
;

modify thing 
    verDoDigIn(actor) = {
        "  ,  << digVerb.sdesc >> << self.thedesc >>. ";
    }
;

singVerb : deepverb, darkVerb
    verb = '' '' '' ''
    sdesc = ""
    action(actor) = {
        "-        ,   , .";
    }
;
    
modify yellVerb
    verb = '' '' '' '' '' '' '' '' '' '' ''
    isDarkVerb = true
    action(actor) = {
	local loc := outerLoc(actor);
	    
        if (defined(loc, &shout))
	    loc.shout(actor);
	else
	    pass action;
    }	   
;

smellVerb : deepverb, darkVerb
    verb = '' '' '' ''
    sdesc = ""
    doAction = 'Smell'

    doDefault(actor, prep, io) = {
        return inherited.doDefault(actor, prep, io) + [air];
    }
;

modify thing
    verDoSmell(actor) = {
        // It is important that this message is printed by verDoSmell 
	// rather than by doSmell: otherwise, the default action on
	// just 'smell' (without direct object) could be to smell
	// whatever uninteresting object was at hand.
        "\^<< self.thedesc >> ,  .";
    }
;

listenVerb : deepverb, darkVerb
    verb = '' '' ' ' ' '
    sdesc = ""
    doAction = 'Listento'

    doDefault(actor, prep, io) = {
        local lst := inherited.doDefault(actor, prep, io);
	
	if (music.state > 0)
	    lst += music;
	return lst;
    }  
;
    
modify thing
    verDoListento(actor) = {
        "\^<< self.thedesc >>   . ";
    }
;

modify doorway
    verDoListento(actor) = { }
    doListento(actor) = {
        "%% %%   . ";
    }
;
    
lightVerb : deepverb
    verb = '' '' '' ''
    sdesc = ""
    isDarkReachableVerb = true 
    doAction = 'Light'
;

extinguishVerb : deepverb
    verb = '' '' '' '' '' '' '' ''
    sdesc = ""
    isDarkReachableVerb = true
    doAction = 'Extinguish'
;

/*
 * Some new system verbs
 */
modify HelpVerb
    verb = '' '' '' '' '' ''
    sdesc = ""
    action(actor) = {
        info.helptext;
	abort;
    }
;

infoVerb : sysverb
    verb = '' ''
    sdesc = ""
    action(actor) = {
        info.infotext;
	      abort;
    }
;

/*
 * And some new abbreviations and synonyms
 */
modify takeVerb
    verb = '' '' '' '' ''
;

modify dropVerb
    verb = '' '' '' '' '' ''
;

modify quitVerb
    verb = ''
;

        
sneezeVerb : deepverb
    verb = '' '' '' ''
    sdesc = ""
    action(actor) = {
        "--! ";
    }
;

playVerb : deepverb, darkVerb
    verb = '' '' '' '' '' ''
    sdesc = ""    
    doAction = 'Play'
;
    
blowVerb : deepverb, darkVerb
    verb = '' ''
    sdesc = ""    
    doAction = 'Blow'
;
    
pourVerb : deepverb
    verb = '' '' '' ''
    sdesc = ""
    doAction = 'Pour'
    ioAction(onPrep) = 'PourOn'
    ioAction(inPrep) = 'PutIn'
;

modify thing
    verIoPourOn(actor) = { }
    ioPourOn(actor, dobj) = {
        dobj.doPourOn(actor, self);
    }
;
    
emptyVerb : deepverb
    verb = '' '' '' ''
    sdesc = ""
    doAction = 'Empty'
;

flipThruVerb : deepverb
    verb = '' ''
    sdesc = ""
    doAction = 'FlipThru'
;
    
modify throwVerb 
    doAction = 'Throw'
    ioAction(thruPrep) = 'ThrowThru'
    ioAction(inPrep) = 'PutIn'
    rejectMultiDobj(prep) = {
        "       . ";
        return true;
    }
;


modify fixeditem 
    verDoThrow(actor) = {
        "%%    << self.thedesc >>. ";
    }
;


/*
 * Allow "attack <dobj>" without any indirect object - silly to ask for
 * one when no weapon is sufficient anyway.
 */
modify attackVerb 
    doAction = 'Attack'
;

modify thing 
    verDoAttack(actor) = {
        "  "; self.thedesc; "   . ";
    }
    verIoAttackWith(actor) = {
        if (not self.isIn(actor))
	    "%% %%  << self.thedesc >>!";
	else
	    "\^<< self.thedesc >>      .";
    }
;

acceptVerb : deepverb
    verb = '' ''
    sdesc = ""
    doAction = 'Take'
    ioAction(fromPrep) = 'TakeOut'
;
    
fillVerb : deepverb
    verb = '' '' '' '' '' '' '' ''
    sdesc = ""
    doAction = 'Fill'
;

modify container
    verDoFill(actor) = {
        "%%   ,      << self.thedesc >>. ";
    }
;

kickVerb : deepverb
    verb = '' ''
    sdesc = ""
    doAction = 'Kick'
;

modify thing
    verDoKick(actor) =
    {
        "  "; self.thedesc; "   . ";
    }
;    
    
    
untwistVerb : deepverb
    verb = '' '' '' ''
    sdesc = ""
    doAction = 'Untwist'
;

/*insertVerb : deepverb
    verb = 'insert'
    sdesc = "insert"
    prepDefault = inPrep
    ioAction(inPrep) = 'PutIn'
    doDefault(actor, prep, io) = {
        return(takeVerb.doDefault( actor, prep, io ) + actor.contents);
    }
    rejectMultiDobj(prep) = {
        "You can't use multiple direct objects with the verb 'insert' "; 
        return true;
    }
;*/

modify putVerb
    rejectMultiDobj(prep) = {
        if (prep <> inPrep)
	    return nil;
	   
        "You can only do that to one thing at a time. ";
        return true;
    }
;
    
modify lookInVerb
    verb = 'look inside' 'l inside' 'gaze into' 
;

modify plugVerb 
    verb = 'plug in'
;

modify wearVerb
    verb = 'don'
;

modify removeVerb
    verb = 'doff'
;

undressVerb : deepverb
    verb = 'undress' 'strip'
    sdesc = "undress"
    action(actor) = {
        "There's no reason why you should undress in this game. ";
    }
    doAction = 'Undress'
;

dressVerb : deepverb
    verb = 'dress'
    sdesc = "dress"
    action(actor) = {
        "%You% will have to tell me what %you% want to wear. ";
    }
    doAction = 'Dress'
;

unwrapVerb : deepverb
    verb = 'unwrap' 'unwind'
    sdesc = "unwrap"
    doAction = 'Unwrap'
;


modify attachVerb 
    verb = 'join'
    doAction = 'Attach'
;

modify detachVerb
    verb = 'separate'
;
    
modify breakVerb
    verb =  'smash'
;

modify thing
    verDoBreak(actor) = {
 	if (not actor.isCarrying(self))
	    "But %you% %are% not holding << self.thedesc >>.";
    }
;
    
modify lookInVerb
    verb = 'look into' 'l into'
;

modify inspectVerb
;

modify turnVerb
    rejectMultiDobj(prep) = {
        "You can only turn one thing at a time. ";
        return true;
    }
;

modify openVerb
    rejectMultiDobj(prep) = {
        "You can only << self.sdesc >> one thing at a time. ";
        return true;
    }
;

modify searchVerb
    rejectMultiDobj(prep) = {
        "You can only << self.sdesc >> one thing at a time. ";
        return true;
    }
;

modify removeVerb
    doDefault(actor, prep, io) = {
    	if (objwords(1) = ['A']) {
            global.allMessage := 'You can\'t use "all" with this verb.';
	    return [];
	}
	pass doDefault;
    }
;



modify knockVerb
    verb = 'knock' 'knock on' 'knock at'
    sdesc = "knock on"
    doAction = 'KnockOn'
    isDarkReachableVerb = true
;

modify thing
    verDoKnockOn(actor) = { 
        "Nothing happens. ";
    }
;

modify doorway
    verDoKnockOn(actor) = {
    }
    doKnockOn(actor) = {
        "There is no answer.";
    }
;
    
modify movableActor
    verDoKnockOn(actor) = {
        "I don't think << self.thedesc >> would appreciate that. ";
    }
;

modify basicMe
    verDoKnockOn(actor) = {
        "Please concentrate on your task, will you?";
    }
;

